#include "canvas.h"
#include <GL/glut.h>
#include "glutMaster.h"
#include "stdlib.h"
#include <cfloat>
#include <time.h>

Canvas::Canvas(int width, int height) : _width(width), _height(height), _picked_particle(0), _draw_shape_functions(true), particles(0), lines(0) {
   // OpenGL initialization
   glutInitDisplayMode (GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   glOrtho(0, width, height, 0, -1, 1);
   glDisable(GL_LIGHTING);
   glDisable(GL_DEPTH_TEST);
   glutInitWindowSize(width,height);
   GlutMaster::instance()->CallGlutCreateWindow("Tutorial Demo Shape Functions", this);

   // Create our particles
   createParticles();

   // Create some lines
   createLines();
}

Canvas::~Canvas() {
   delete particles;
   delete lines;
}


void Canvas::CallBackDisplayFunc() {
   // Clear the background with the 0 color
   vector3f color(240,1,1);
   color.HSVtoRGB();
   glClearColor(color.x,color.y,color.z,0);
   glClear(GL_COLOR_BUFFER_BIT);

   // Draw the shape/weight function of the picked particle
   particles->getParticle(_picked_particle).draw(particles, lines, _draw_shape_functions);

   // Draw the particle positoins
   particles->draw();

   // Draw the crack lines
   glColor3f(1,1,1);
   glLineWidth(4);
   glBegin(GL_LINES);
   for (int i=0; i<lines->nbLines(); i++) {
      LineSegment *l = lines->getLine(i);
      glVertex3fv(&(l->getP0().x));
      glVertex3fv(&(l->getP1().x));
   }
   glEnd();

   glutSwapBuffers();
}

void Canvas::CallBackMouseFunc(int button, int state, int x, int y) {
   // if we click with the left button,
   // we draw the shape functions, if we
   // click with the right button, we draw
   // the weight functions instead.
   if (button == GLUT_LEFT_BUTTON) {
      _draw_shape_functions = true;
   }
   else if (button == GLUT_RIGHT_BUTTON) {
      _draw_shape_functions = false;
   }

   // pick closest particle
   if (state == GLUT_DOWN) {
      int min_index = -1;
      float min_dist = FLT_MAX;
      vector3f pos(640.0f*x/_width,480.0f*y/_height,0);
      for (int i=0; i<particles->nbParticles(); i++) {
         const Particle &p = particles->getParticle(i);
         float dist = p.distance(pos);
         if (dist<min_dist) {
            min_index = i;
            min_dist = dist;
         }
      }
      _picked_particle = min_index;
   }
   glutPostRedisplay();
   
}

void Canvas::CallBackKeyboardFunc(unsigned char key, int x, int y) {
   switch(key) {
      case 'q': exit(0); break;
      case 'p': createParticles(); break;
      case 'd': _draw_shape_functions = !_draw_shape_functions; break;
   }
   glutPostRedisplay();
}
      
void Canvas::CallBackReshapeFunc(int w, int h) {
   _width = w;
   _height = h;
   glViewport(0,0,_width,_height);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   glOrtho(0, 640, 480, 0, -1, 1);
}

void Canvas::createParticles() {
   delete particles;
   srand(time(NULL));
   particles = new Particles();
   for (int x=-90; x<_width+90; x+=40) {
      for (int y=-90; y<_height+90; y+=40) {
         int randx = (rand()/float(RAND_MAX)-0.5f)*30;
         int randy = (rand()/float(RAND_MAX)-0.5f)*30;
         particles->addParticle(vector3f(x+randx,y+randy,0));
      }
   }
   //particles->setParticleNeighbors();
   //particles->evaluateShapeFunctions();
   //particles->evaluateGradientShapeFunctions();
}

void Canvas::createLines() {
   lines = new Lines();

   const vector3f p1(-640,240,0);
   const vector3f p2(200,240,0);
   LineSegment * l1 = new LineSegment(p1,p2,true);
   lines->addLine(l1);

   const vector3f p3(640-200,240,0);
   const vector3f p4(1000,240,0);
   LineSegment * l2 = new LineSegment(p3,p4,false);
   lines->addLine(l2);
}
