#include "lineSegment.h"


bool LineSegment::intersection(const LineSegment &other, float &t, vector3f &p) const {
   // threshold value used below
   const float myeps = 0.00000001f;

   const vector3f &p1 = getP0();
   const vector3f &p2 = getP1();

   const vector3f &p3 = other.getP0();
   const vector3f &p4 = other.getP1();

   // use cross product implementation in vector3f for ease of use
   const float denom = ((p3-p4).cross(p2-p1)).z;

   // make sure we are not dividing by zero
   if (fabs(denom)<myeps) return false;

   const float t1 = ((p4-p3).cross(p1-p3)).z / denom;
   const float t2 = ((p2-p1).cross(p1-p3)).z / denom;

   // is the intersection point on both segments?
   if (t1>=0 && t1<=1 && t2>=0 && t2<=1) {
      t = t1;
      p = p1+t1*(p2-p1);
      return true;
   }
   else return false;
}
