#ifndef __LINESEGMENT_H__
#define __LINESEGMENT_H__

#include "vector3f.h"

/*
 * Represents a line segment in 2D
 *
 * Note that although vector3f is used,
 * the calculations in the intersection
 * method assume the line segment is in
 * 2D and only the x and y coordinates
 * are used.
 *
 * A linesegment can be transparent or not.
 * If transparent, p1 is always the tip.
 * See tutorial for more details.
 */
class LineSegment {

   public:

      // Create a line segment from p0 to p1, p1 is the tip
      LineSegment(const vector3f &p0, const vector3f &p1, bool transparent=false) : _p0(p0), _p1(p1), _transparent(transparent) { }

      // Compute intersection between this and the given line segment
      // Returns true if there is an intersection, false otherwise
      // Also returns the t-parameter value (should be between 0 and 1 and
      // to be used from this->_p0 to this->_p1. 
      // Also returns the actual intersection point p.
      bool intersection(const LineSegment &other, float &t, vector3f &p) const;
      
      // Returns the first point of the line segment
      inline const vector3f& getP0() const { return _p0; }

      // Returns the second point of the line segment
      inline const vector3f& getP1() const { return _p1; }

      // Returns the tip of the line segment
      inline const vector3f& getTip() const { return _p1; }
   
      // Is this line segment transparent?
      inline bool isTransparent() const { return _transparent; }

   private:

      vector3f _p0;
      vector3f _p1; // if transparent, this is the tip

      bool _transparent;
};

#endif
