#ifndef __MATRIX_H__
#define __MATRIX_H__

#include <iostream>
#include "matrix3x3f.h"
using namespace std;

class MyMatrix {
   public: 

      MyMatrix(int nbr, int nbc) {
         nbrows = nbr;
         nbcolumns = nbc;

         A = new double[nbrows*nbcolumns];
         copyA = new double[nbrows*nbcolumns];
         perm = new int[nbrows];
         result = new double[nbrows];
         for (int i=0; i<nbrows; i++) {
            perm[i] = i;
         }
         for (int i=0; i<nbrows; i++) {
            for (int j=0; j<nbcolumns; j++) {
               A[i+j*nbrows] = 0;
               copyA[i+j*nbrows] = 0;
            }
         }
      }

      MyMatrix(MyMatrix &copy) {
         nbrows = copy.nbrows;
         nbcolumns = copy.nbcolumns;

         A = new double[nbrows*nbcolumns];
         copyA = new double[nbrows*nbcolumns];
         perm = new int[nbrows];
         result = new double[nbrows];
         for (int i=0; i<nbrows; i++) {
            perm[i] = copy.perm[i];
         }
         for (int i=0; i<nbrows; i++) {
            for (int j=0; j<nbcolumns; j++) {
               A[i+j*nbrows] = copy.A[i+j*nbrows];
               copyA[i+j*nbrows] = copy.copyA[i+j*nbrows];
            }
         }
      }

      ~MyMatrix() {
         delete[] result;
         delete[] A;
         delete[] copyA;
         delete[] perm;
      }

      void set(int row, int column, double element) {
         //if (row<nbrows && column <nbcolumns) {
            A[row+nbrows*column] = element;
            copyA[row+nbrows*column] = element;
         //}
      }

      void add(int row, int column, double element) {
         //if (row<nbrows && column <nbcolumns) {
            A[row+nbrows*column] += element;
            copyA[row+nbrows*column] += element;
         //}
      }

      double& operator () (int row, int column) {
         return A[row+nbrows*column];
      }

      void addSymmetric(int row, int column, double element) {
         //if (row<nbrows && column <nbcolumns) {
            A[row+nbrows*column] += element;
            if (row!=column) A[column+nbrows*row] += element;
            copyA[row+nbrows*column] += element;
            if (row!=column) copyA[column+nbrows*row] += element;
         //}
      }

      double get(int row, int column) {
         return A[row+nbrows*column];
      }

      // shuffle so that the max'th element in the row-th row at column row
      bool shuffle(int row) {
         int column = row;
         int maxrow = -1;
         double maxabsvalue = 0;
         for (int i=0; i<nbrows; i++) {
            int index = i+nbrows*column;
            if (abs(A[index]) > maxabsvalue) {
               maxabsvalue = abs(A[index]);
               maxrow = i;
            }
         }
         if (maxrow==-1 || maxabsvalue==0) return false;
         // swap maxrow and row
         if (maxrow!=row) {
            for (int i=0; i<nbcolumns; i++) {
               double temp = A[row+nbrows*column];
               A[row+nbrows*column] = A[maxrow+nbrows*column];
               A[maxrow+nbrows*column] = temp;
            }
            int temp = perm[row];
            perm[row] = perm[maxrow];
            perm[maxrow] = temp;
         }
         return true;
      }

      // make element at row,row 1
      bool normalize(int row) {
         int column = row;
         double value = A[row+nbrows*column];
         if (value==0) return false;
         double inv_value = 1.0/value;
         for (int i=0; i<nbcolumns; i++) {
            A[row+nbrows*i]*=inv_value;
         }
         return true;
      }

      // subtract row from all other rows
      void subtract(int row) {
         int column = row;
         for (int i=row+1; i<nbrows; i++) {
            if (i==row) continue;
            double value = A[i+nbrows*column];
            for (int j=0; j<nbcolumns; j++) {
               A[i+nbrows*j]-=value*A[row+nbrows*j];
            }
         }
      }

      bool gausseidel() {
         for (int i=0; i<nbrows; i++) {
            //if (!shuffle(i)) return false;
            if (normalize(i)) subtract(i);      
         }
         return backsubstitute();
      }

      bool backsubstitute() {
         for (int i = nbrows-1; i>=0; i--) {
            double b = A[i+nbrows*(nbrows)];
            for (int k=i+1; k<nbrows; k++) {
               b-=A[i+nbrows*k]*result[k];
            }
            result[i] = b;
         }
         return true;
      }

      void print() {
         for (int i=0; i<nbrows; i++) {
            for (int j=0; j<nbcolumns; j++) {
               cout << A[i+nbrows*j] << "   ";
            }
            //cout << endl;
            cout << " | " << perm[i] << endl;
         }
         cout << endl;
         //check();
      }

      void check() {
         for (int i=0; i<nbrows; i++) {
            double value = 0;
            for (int j=0; j<nbcolumns-1; j++) {
               value+=copyA[i+nbrows*j]*result[j];
            }
            cout << value << " ?=? " << copyA[i+nbrows*(nbcolumns-1)] << endl;
         }
      }

      void unscramble() {
         double *temp = new double[nbrows];
         for (int i=0; i<nbrows; i++) {
            temp[i] = A[perm[i]+nbrows*(nbcolumns-1)];
         }
         for (int i=0; i<nbrows; i++) {
            A[i+nbrows*(nbcolumns-1)] = temp[i];
         }
      }

      matrix3x3f invert() {
         matrix3x3f invM; invM.zero();
         for (int i=0; i<3; i++) {
            MyMatrix help(*this);
            for (int j=0; j<3; j++) {
               if (j==i) help(j,3) = 1;
               else help(j,3) = 0;
            }
            help.gausseidel();
            for (int k=0; k<3; k++) invM(k,i) = help.result[k];
         }  
         return invM;
      }

      int nbrows;
      int nbcolumns;
      double * result;

      private:
      double *A;
      double *copyA;
      int *perm;  

};

#endif
