#include "matrix3x3f.h"
#include "matrix.h"
#include <cmath>
#include <iostream>
using namespace std;

matrix3x3f::matrix3x3f(MyMatrix * mm) {
      for (int i=0; i<3; i++) {
         for (int j=0; j<3; j++) {
            (*this)(i,j) = (*mm)(i,j);
         }
      }
   }



matrix3x3f::matrix3x3f( float m0, float m3, float  m6,
                         float m1, float m4, float  m7,
                         float m2, float m5, float  m8)
{
	m[0]=m0; m[3]=m3; m[6] =m6;
	m[1]=m1; m[4]=m4; m[7] =m7;
	m[2]=m2; m[5]=m5; m[8] =m8;
}

void matrix3x3f::add(  float m0, float m3, float  m6,
                         float m1, float m4, float  m7,
                         float m2, float m5, float  m8)
{
	m[0]+=m0; m[3]+=m3; m[6]+=m6;
	m[1]+=m1; m[4]+=m4; m[7]+=m7;
	m[2]+=m2; m[5]+=m5; m[8]+=m8;
}

void matrix3x3f::identity( void ) {
	m[0]=1.0f; m[3]=0.0f; m[6] =0.0f;
	m[1]=0.0f; m[4]=1.0f; m[7] =0.0f;
	m[2]=0.0f; m[5]=0.0f; m[8]= 1.0f;
}

void matrix3x3f::print() const {
   for (int i=0; i<3; i++) {
      for (int j=0; j<3; j++) {
         cout << m[i+3*j] << " ";
      }
      cout << endl;
   }
}

void matrix3x3f::transform( vector3f *vec ) const {
   vector3f &v = *vec;

   float x = v.x;
   float y = v.y;
   float z = v.z;

	v.x = x * m[0] +
         y * m[3] +
         z * m[6];

   v.y = x * m[1] +
         y * m[4] +
         z * m[7];

   v.z = x * m[2] +
         y * m[5] +
         z * m[8];
}

matrix3x3f matrix3x3f::operator + ( const matrix3x3f &other ) {
	matrix3x3f result;

	result.m[0]  = m[0]  + other.m[0];
	result.m[1]  = m[1]  + other.m[1];
	result.m[2]  = m[2]  + other.m[2];
	result.m[3]  = m[3]  + other.m[3];

	result.m[4]  = m[4]  + other.m[4];
	result.m[5]  = m[5]  + other.m[5];
	result.m[6]  = m[6]  + other.m[6];
	result.m[7]  = m[7]  + other.m[7];

	result.m[8]  = m[8]  + other.m[8];

	return result;
}

matrix3x3f matrix3x3f::transpose() const {
   matrix3x3f result;
   
   result.m[0]  = m[0];  
   result.m[1]  = m[3];
   result.m[2]  = m[6];
   result.m[3]  = m[1];
   result.m[4]  = m[4];
   result.m[5]  = m[7];
   result.m[6]  = m[2];
   result.m[7]  = m[5];
   result.m[8]  = m[8];

   return result;
}

matrix3x3f matrix3x3f::operator - ( const matrix3x3f &other ) {
	matrix3x3f result;

	result.m[0]  = m[0]  - other.m[0];
	result.m[1]  = m[1]  - other.m[1];
	result.m[2]  = m[2]  - other.m[2];
	result.m[3]  = m[3]  - other.m[3];
	result.m[4]  = m[4]  - other.m[4];
	result.m[5]  = m[5]  - other.m[5];
	result.m[6]  = m[6]  - other.m[6];
	result.m[7]  = m[7]  - other.m[7];
	result.m[8]  = m[8]  - other.m[8];

	return result;
}

matrix3x3f matrix3x3f::operator * ( const matrix3x3f &other ) const {
	matrix3x3f result;

	result.m[0]  = (m[0]*other.m[0])+(m[3]*other.m[1])+(m[6]*other.m[2]);
   result.m[1]  = (m[1]*other.m[0])+(m[4]*other.m[1])+(m[7]*other.m[2]);
	result.m[2]  = (m[2]*other.m[0])+(m[5]*other.m[1])+(m[8]*other.m[2]);

	result.m[3]  = (m[0]*other.m[3])+(m[3]*other.m[4])+(m[6]*other.m[5]);
   result.m[4]  = (m[1]*other.m[3])+(m[4]*other.m[4])+(m[7]*other.m[5]);
	result.m[5]  = (m[2]*other.m[3])+(m[5]*other.m[4])+(m[8]*other.m[5]);

	result.m[6]  = (m[0]*other.m[6])+(m[3]*other.m[7])+(m[6]*other.m[8]);
   result.m[7]  = (m[1]*other.m[6])+(m[4]*other.m[7])+(m[7]*other.m[8]);
	result.m[8]  = (m[2]*other.m[6])+(m[5]*other.m[7])+(m[8]*other.m[8]);

	return result;
}

matrix3x3f matrix3x3f::operator * ( const float scalar ) {
	matrix3x3f result;

	result.m[0]  = m[0]  * scalar;
	result.m[1]  = m[1]  * scalar;
	result.m[2]  = m[2]  * scalar;
	result.m[3]  = m[3]  * scalar;
	result.m[4]  = m[4]  * scalar;
	result.m[5]  = m[5]  * scalar;
	result.m[6]  = m[6]  * scalar;
	result.m[7]  = m[7]  * scalar;
	result.m[8]  = m[8]  * scalar;
	return result;
}

float matrix3x3f::getInfNorm() const {
  //float result = 0.0f;
   float n1 = fabs(m[0])+fabs(m[3])+fabs(m[6]);
   float n2 = fabs(m[1])+fabs(m[4])+fabs(m[7]);
   float n3 = fabs(m[2])+fabs(m[5])+fabs(m[8]);
   return max(n1, max(n2,n3));
}


float matrix3x3f::getDeterminant() const {
   return m[0]*m[4]*m[8]+m[2]*m[3]*m[7]+m[1]*m[5]*m[6]
         -m[2]*m[4]*m[6]-m[1]*m[3]*m[8]-m[0]*m[5]*m[7];
}
