#include "particles.h"
#include <GL/glut.h>
#include <iostream>
#include "lines.h"
using namespace std;

void Particles::draw() {
   const int nb = nbParticles();
   glColor3f(1,1,1);
   glEnable(GL_POINT_SMOOTH);
   glPointSize(6);
   glBegin(GL_POINTS);
   for (int i=0; i<nb; i++) {
      const vector3f p = getParticle(i);
      glVertex2f(p[0], p[1]);
   }
   glEnd();
}

void Particles::setParticleNeighbors(Lines * lines) {
   cout << "Computing particle neighbors" << endl;
   const int nb = nbParticles();
   for (int i=0; i<nb; i++) {
      Particle &pi = getParticle(i);
      pi.setParticleNeighbors(this, lines);
      cout << "   Particle " << i << " has " << pi.nbNeighbors() << " neighbors " << endl;
   }
   cout << "Computing particle neighbors done" << endl;
}

void Particles::evaluateShapeFunctions() {
   cout << "Evaluating shape functions" << endl;
   const int nb = nbParticles();
   for (int i=0; i<nb; i++) {
      Particle &pi = getParticle(i);
      pi.evaluateShapeFunctions(this);
   }
   cout << "Evaluating shape functions done" << endl;
}

void Particles::evaluateGradientShapeFunctions() {
   cout << "Evaluating gradient shape functions" << endl;
   const int nb = nbParticles();
   for (int i=0; i<nb; i++) {
      Particle &pi = getParticle(i);
      pi.evaluateGradientShapeFunctions(this);
   }
   cout << "Evaluating gradient shape functions done" << endl;
}
