#ifndef __PARTICLES_H__
#define __PARTICLES_H__

#include "particle.h"
#include <vector>
using namespace std;

class Lines;

/**
 * A collection of particles
 */
class Particles {
   public:
      // The only parameter is the default support
      // radius that should be used when adding a
      // particle and no support radius is given.
      Particles(float h=100) :  _h(h) {}

      // Add a particle at the given position with the
      // default support radius.
      inline void addParticle(const vector3f &pos) { 
         addParticle(pos, _h); 
      }

      // Add a particle at the given position with the
      // given support radius. The particle gets as ID the index
      // in the particle vector.
      inline void addParticle(const vector3f &pos, const float h) {
         int index = _particles.size();
         _particles.push_back(Particle(pos,h,index));
      
      }

      // Number of particles in this collection.
      inline int nbParticles() const { return _particles.size(); }

      // Return the particle at the given position.
      inline Particle& getParticle(const int index) { return _particles[index]; }

      // Return the particle at the given position.
      inline const Particle& getParticle(const int index) const { return _particles[index]; }

      // Brute force particle neighbors computation.
      // This can be accelerated using one of the data
      // structures discussed in the tutorial
      void setParticleNeighbors(Lines * lines = 0);

      // Compute the shape functions for the particle neighbors
      void evaluateShapeFunctions();
  
      // Compute the spatial gradients of the shape functions
      void evaluateGradientShapeFunctions();

      // Draw the particles using OpenGL
      void draw();

   private:

      vector<Particle> _particles; // vector of particles

      float _h; // default support radius
};

#endif
