#ifndef __VECTOR3F_H__
#define __VECTOR3F_H__

#include <fstream>
#include <iostream>
#include <cmath>
#include <cstdlib>
using namespace std;

//#define min(A,B) (A<B?A:B)
//#define max(A,B) (A>B?A:B)

class vector3f {
public:

   union {
      float v[3];
      struct {
         float x;
         float y;
         float z;
      };
      struct {
         float r;
         float g;
         float b;
      };
   };

   vector3f() : x(0), y(0), z(0) { }
   vector3f( float x_, float y_, float z_ ) : x(x_), y(y_), z(z_) { }

inline void zero() {
      x = 0.0f;
      y = 0.0f;
      z = 0.0f;
   }


inline vector3f(ifstream &input, bool ascii) {
   read(input, ascii);
}

inline void setRandom() {
   x = float(rand())/float(RAND_MAX);
   y = float(rand())/float(RAND_MAX);
   z = float(rand())/float(RAND_MAX);
}

inline void abs() {
   x = fabs(x);
   y = fabs(y);
   z = fabs(z);
}

inline void  set( float x_, float y_, float z_ ) {
   x = x_;
   y = y_;
   z = z_;
}

inline void  set(float *v) {
   x = v[0];
   y = v[1];
   z = v[2];
}

inline void  print() const {
   cout << x << "," << y << "," << z << endl;
}

inline vector3f  perpendicular() const {
   vector3f result;
   if (fabs(x) < fabs(y)) {
      if (fabs(x) < fabs(z)) {
         result.x = 1.0f-x*x;
         result.y = -x*y;
         result.z = -x*z;
         return result;
      }
   }
   else {
      if (fabs(y) < fabs(z)) {
         result.x  =-y*x;
         result.y = 1.0f-y*y;
         result.z = -y*z;
         return result;
      }
   }
   result.x = -z*x;
   result.y = -z*y;
   result.z = 1.0f-z*z;
   return result;
}

 vector3f rotateZ(float theta){
   vector3f result;
   
   
   result.x = cos(theta)*x - sin(theta)*y;
   result.y = sin(theta)*x + cos(theta)*y;
   result.z = z;

   return result;
 }



void  write(ofstream &output, bool ascii) const {
   //writeFloat(output, x, ascii);
   //writeFloat(output, y, ascii);
   //writeFloat(output, z, ascii);
}

void  read(ifstream &input, bool ascii) {
   //x = readFloat(input, ascii);
   //y = readFloat(input, ascii);
   //z = readFloat(input, ascii);
}

inline float length() const {
   return( (float)sqrt( x * x + y * y + z * z ) );
}

inline float getSquaredLength() const {
     return( x * x + y * y + z * z  );
}

inline float normalize() {
   float fLength = length();
      x = x / fLength;
      y = y / fLength;
      z = z / fLength;
   return fLength;
}


inline float distance(const vector3f &v2) const {
   float dx = x - v2.x;
   float dy = y - v2.y;
   float dz = z - v2.z;
   return (float)sqrt( dx * dx + dy * dy + dz * dz );
}

inline float distanceSquared(const vector3f &v2) const {
   float dx = x - v2.x;
   float dy = y - v2.y;
   float dz = z - v2.z;
   return ( dx * dx + dy * dy + dz * dz );
}


inline vector3f  crossProduct(const vector3f &v2 ) const {
   vector3f vCrossProduct;
   vCrossProduct.x =  y * v2.z - z * v2.y;
   vCrossProduct.y = -x * v2.z + z * v2.x;
   vCrossProduct.z =  x * v2.y - y * v2.x;
   
   return vCrossProduct;
}

inline vector3f  cross(const vector3f &v2 ) const {
   vector3f vCrossProduct;
   vCrossProduct.x =  y * v2.z - z * v2.y;
   vCrossProduct.y = -x * v2.z + z * v2.x;
   vCrossProduct.z =  x * v2.y - y * v2.x;
   
   return vCrossProduct;
}

inline vector3f  vmin(const vector3f &v2) const {
   vector3f result;
   result.x = (x<v2.x? x: v2.x);
   result.y = (y<v2.y? y: v2.y);
   result.z = (z<v2.z? z: v2.z);
   return result;
}

inline vector3f  vmax(const vector3f &v2) const {
   vector3f result;
   result.x = (x>v2.x? x: v2.x);
   result.y = (y>v2.y? y: v2.y);
   result.z = (z>v2.z? z: v2.z);
   return result;
}

inline float  maxnorm(const vector3f &v2) const {
   float dx = v2.x-x; if (dx<0) dx=-dx;
   float dy = v2.y-y; if (dy<0) dy=-dy;
   float dz = v2.z-z; if (dz<0) dz=-dz;
   float maxim = max(dx,dy);
   maxim = max(maxim,dz);
   
   return maxim;
}

// Operators...

inline vector3f  operator + ( const vector3f &other ) const {
   vector3f vResult(0.0f, 0.0f, 0.0f);
   
   vResult.x = x + other.x;
   vResult.y = y + other.y;
   vResult.z = z + other.z;
   
   return vResult;
}

inline vector3f  operator + ( const float scalar ) const {
   vector3f vResult;
   
   vResult.x = x + scalar;
   vResult.y = y + scalar;
   vResult.z = z + scalar;
   
   return vResult;
}
inline vector3f  operator + ( void ) const {
   return *this;
}

inline float& operator[] (const int index) {
   return v[index];
}
inline float  operator[] (const int index) const {
   return v[index];
}

inline void set(int index, float value) {
   if (index==0) x = value;
   else if (index==1) y = value;
   else if (index==2) z = value;
}
inline vector3f  operator - ( const vector3f &other ) const {
   vector3f vResult(0.0f, 0.0f, 0.0f);
   
   vResult.x = x - other.x;
   vResult.y = y - other.y;
   vResult.z = z - other.z;
   
   return vResult;
}

inline vector3f  operator - (const float value ) const {
   vector3f vResult(0.0f, 0.0f, 0.0f);
   
   vResult.x = x - value;
   vResult.y = y - value;
   vResult.z = z - value;
   
   return vResult;
}



inline void bound(const vector3f &lower, const vector3f &upper) {
   if (x<lower.x) x = lower.x;
   else if (x>upper.x) x = upper.x;
   if (y<lower.y) y = lower.y;
   else if (y>upper.y) y = upper.y;
   if (z<lower.z) z = lower.z;
   else if (z>upper.z) z = upper.z;
}

inline vector3f  operator - ( void ) const {
   vector3f vResult(-x, -y, -z);
   
   return vResult;
}

inline vector3f  operator * ( const vector3f &other ) const {
   vector3f vResult;
   
   vResult.x = x * other.x;
   vResult.y = y * other.y;
   vResult.z = z * other.z;
   
   return vResult;
}

inline vector3f operator * ( const float scalar ) const {
   return vector3f(x*scalar, y*scalar, z*scalar);
}

vector3f inline operator / ( const float s ) const {
   vector3f vResult(0.0f, 0.0f, 0.0f);
   float scalar = 1.0f/s;
   
   vResult.x = x * scalar;
   vResult.y = y * scalar;
   vResult.z = z * scalar;
   
   return vResult;
}


//friend vector3f operator * (const float scalar, const vector3f &other);

friend vector3f operator * ( const float scalar, const vector3f &other ) {
   return vector3f(other.x*scalar, other.y*scalar, other.z*scalar);
}

inline vector3f operator / ( const vector3f &other ) const {
   vector3f vResult(0.0f, 0.0f, 0.0f);
   
   vResult.x = x / other.x;
   vResult.y = y / other.y;
   vResult.z = z / other.z;
   
   return vResult;
}

inline vector3f& operator = ( const vector3f &other ) {
   x = other.x;
   y = other.y;
   z = other.z;
   
   return *this;
}

inline vector3f& operator += ( const vector3f &other ) {
   x += other.x;
   y += other.y;
   z += other.z;
   
   return *this;
}

inline vector3f& operator *= ( const vector3f &other ) {
   x *= other.x;
   y *= other.y;
   z *= other.z;
   
   return *this;
}

inline vector3f& operator *= ( const float scalar ) {
   x *= scalar;
   y *= scalar;
   z *= scalar;
   
   return *this;
}

inline vector3f& operator /= ( const float scalar ) {
  return operator*=(1.f/scalar);
}

inline vector3f& operator -= ( const vector3f &other ) {
   x -= other.x;
   y -= other.y;
   z -= other.z;
   
   return *this;
}

inline vector3f& operator -= ( const float scalar) {
   x -= scalar;
   y -= scalar;
   z -= scalar;
   
   return *this;
}

inline vector3f& operator += ( const float scalar) {
   x += scalar;
   y += scalar;
   z += scalar;
   
   return *this;
}

inline bool equals(const vector3f &s) const {
   return (fabs(x-s.x)<0.0000001)&&(fabs(y-s.y)<0.0000001)&&(fabs(z-s.z)<0.00000001);
}


inline void colorize() {
   if (x>1 || y>1 || z>1)  {
      x/=255.0f;
      y/=255.0f;
      z/=255.0f;
   }
}

inline void set(const vector3f &other) {
      x = other.x;
      y = other.y;
      z = other.z;
}

inline float dotProduct(const vector3f &v2 ) const {
      return( x * v2.x + y * v2.y + z * v2.z  );
   }
inline float dot(const vector3f &v2 ) const {
   return( x * v2.x + y * v2.y + z * v2.z  );
}

inline void vceil() {
   x = ceil(x);
   y = ceil(y);
   z = ceil(z);
}

inline void vfloor() {
   x = floor(x);
   y = floor(y);
   z = floor(z);
}



void HSVtoRGB()
{
   float h = x;
   float s = y;
   float v = z;
   float *rr = &x;
   float *gg = &y;
   float *bb = &z;
   int i;
   float f, p, q, t;

   if( s == 0 ) {
      // achromatic (grey)
      *rr = *gg = *bb = v;
      return;
   }

   h /= 60;       // sector 0 to 5
   i = (int)floor( h );
   f = h - i;        // factorial part of h
   p = v * ( 1 - s );
   q = v * ( 1 - s * f );
   t = v * ( 1 - s * ( 1 - f ) );

   switch( i ) {
      case 0:
         *rr = v;
         *gg = t;
         *bb = p;
         break;
      case 1:
         *rr = q;
         *gg = v;
         *bb = p;
         break;
      case 2:
         *rr = p;
         *gg = v;
         *bb = t;
         break;
      case 3:
         *rr = p;
         *gg = q;
         *bb = v;
         break;
      case 4:
         *rr = t;
         *gg = p;
         *bb = v;
         break;
      default:    // case 5:
         *rr = v;
         *gg = p;
         *bb = q;
         break;
   }

}

};



std::ostream &operator<<(std::ostream &os, vector3f const &v);
std::istream &operator>>(std::istream &is, vector3f &v);
#endif
